﻿AlchemyList = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceHook-2.1")
local L = AceLibrary("AceLocale-2.2"):new("AlchemyList")
local T = AceLibrary("Gratuity-2.0")

local chatprefix2 = "<AlchemyList>"
local LOCKED = false
local vmajor, vminor = "AlchemyList-1.0", "$Rev: 62889 $"

AlchemyList.options = {
	type="group",
	args = {
		inc = {
			type = "toggle",
			name = L["Show incoming requests"],
			desc = L["Show incoming requests"],
			get = function() return AlchemyList.db.account.showincmessages end,
			set = function(v) AlchemyList.db.account.showincmessages = v end,
		},
		out = {
			type="toggle",
			name = L["Show your replies"],
			desc = L["Show your replies"],
			get = function() return AlchemyList.db.account.showexmessages end,
			set = function(v) AlchemyList.db.account.showexmessages = v end,
		},
		autoscan = {
			type = "toggle",
			name = L["Autoscan"],
			desc = L["Automatically scan your Alchemy panel when it opens"],
			get = function() return AlchemyList.db.account.autoscan end,
			set = "ToggleAutoScan",
		},
		desc = {
			type="toggle",
			name = L["Search description"],
			desc = L["Also search alchemy description for a match"],
			get = function() return AlchemyList.db.account.searchdesc end,
			set = function(v) AlchemyList.db.account.searchdesc = v end,
		},
		level = {
			type="range",
			name = L["Minimum use level"],
			desc = L["Restrict AlchemyList output to a minimum level to use"],
			get = function() return AlchemyList.db.account.level end,
			set = function(v) AlchemyList.db.account.level = v end,
			min = 0,
			max = 300,
			step = 1,
		},
		scan = {
			type="execute",
			name = L["Manual scan (open your Alchemy panel first)"],
			desc = L["Manual scan (open your Alchemy panel first)"],
			func = "Scan",
			order = 101,
		},
		tlock = {
			type="range",
			name = L["Lock time between 2 requests (antispam)"],
			desc = L["Lock time between 2 requests (antispam)"],
			get = function() return AlchemyList.db.account.TIMELOCK end,
			set = function(v) AlchemyList.db.account.TIMELOCK = v end,
			min = 1,
			max = 30,
			step = 1,
		},
		chatprefix = {
			type = "text",
			name = L["Chat prefix to trigger AlchemyList"],
			desc = L["Chat prefix to trigger AlchemyList"],
			get = "getChatPrefix",
			set = function(v) AlchemyList.db.account.chatprefix = v end,
			usage = L["<prefix>"]
		},
		send = {
			type = "text",
			name = L["Send a AlchemyList to a player"],
			desc = L["Send a AlchemyList to a player"],
			get = false,
			set = "Send",
			usage = L["<playername> [query]"]
		},
		show = {
			type = "text",
			name = L["Show a AlchemyList to your local chat window"],
			desc = L["Show a AlchemyList to your local chat window"],
			get = false,
			set = "Show",
			usage = L["[query]"]
		},
		version = {
			type = "execute",
			name = "Version: ",
			desc = vmajor .. " " .. vminor,
			func = function() AlchemyList:Print(vmajor .. " " .. vminor) end,
			order = 102,
		},
	}
}

function AlchemyList:OnInitialize()
	self:Reset()
	self:RegisterDB("AlchemyListDB")
	self:RegisterDefaults("account", {
		Alchemy = {},
		AlchemyName = {},
		AlchemyDesc = {},
		MinLevel = {},

		showincmessages = false,
		showexmessages = false,
		searchdesc = true,
		autoscan = true,
		flask = false,
		potion = false,
		transmutation = false,
		level = 20,
		TIMELOCK = 3,
		chatprefix = "!alchy"
		}
	)
	self:RegisterChatCommand({"/alist", "/alchylist"}, self.options)
end

function AlchemyList:OnEnable()
	self:Hook("ChatFrame_MessageEventHandler", "WhisperHandler", true)
	self:RegisterEvent("CHAT_MSG_WHISPER")
	if self.db.account.autoscan then
		self:RegisterEvent("TRADE_SKILL_SHOW")
	end
end

function AlchemyList:OnDisable()
	self:Reset()
	self:UnhookAll()
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:UnregisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("TRADE_SKILL_SHOW")
	self:UnregisterEvent("TRADE_SKILL_SHOW")
end

function AlchemyList:Send(arg1)
	local argum = self:GetArgs(arg1, " ")
	local sender = argum[1]

	table.remove(argum, 1)

	if table.getn(argum) == 0 then
		self:printHelp(sender)
		return
	end

	self:Handle(argum, sender)
end

function AlchemyList:Show(arg1)
	local argum = self:GetArgs(arg1, " ")
	local sender = "AlchemyListChatOutput"

	if table.getn(argum) == 0 then
		self:Print("No query given.")
		return
	end

	self:Handle(argum, sender)
end

function AlchemyList:Reset()
	LOCKED = false
end

function AlchemyList:WhisperHandler(event)
	if arg1 and arg1 ~= "" then
		if (string.sub(arg1, 1, string.len(self:getChatPrefix())) == self:getChatPrefix() and
		not self.db.account.showincmessages and
		event == "CHAT_MSG_WHISPER") then
			return
		end

		if (string.sub(arg1, 1, string.len(chatprefix2)) == chatprefix2 and
		not self.db.account.showexmessages and
		event == "CHAT_MSG_WHISPER_INFORM") then
			return
		end
	end
	return self.hooks["ChatFrame_MessageEventHandler"](event)
end

function AlchemyList:printHelp(sender)
	if LOCKED then
		self:Respond(L["Wait ... ("]..self.db.account.TIMELOCK..L[" seconds lock (antispam))"], sender)
		return
	end

	local numLines = tonumber(L["numHelpLines"])

	self:Respond(L["usage1"], sender)
	for i = 2, numLines do
		self:Respond(self:getChatPrefix() .. L["usage"..i], sender)
	end

	self:Respond(L["examples1"], sender)
	for i = 2, numLines do
		self:Respond(self:getChatPrefix() .. L["examples"..i], sender)
	end

	if self.db.account.searchdesc then
		self:Respond(L["Searching alchemy description is currently enabled."], sender)
	else
		self:Respond(L["Searching alchemy description is currently disabled."], sender)
	end

	self:Lock()
end

function AlchemyList:CHAT_MSG_WHISPER(...)
	local argum = self:GetArgs(select(1, ...), " ")
	local sender = select(2, ...)
	local tag = select(6, ...)

	if argum[1] and (argum[1] == self:getChatPrefix() or strfind(argum[1], "!alchy")) then
		if tag == L["AFK"] or tag == L["DND"] then
			self:Respond(L["First clear your tag: "] .. tag, sender)
			return
		end

		table.remove(argum, 1)

		if table.getn(argum) == 0 then
			self:printHelp(sender)
			return
		end

		self:Handle(argum, sender)
	end
end

function AlchemyList:GetArgs(text, separator)
	-- capture a link
	-- only match itemID
	local found, _, itemString = string.find(text, "\124%x+\124H(item:%d+):.+\124h%[.+%]")
	text = string.gsub(text, "\124c.+\124r", "")

	-- copy arguments to table
	local args = {}
	local i = 0

	for value in string.gmatch(text, "[^"..separator.."]+") do
		i = i + 1
		args[i] = string.lower(value)
	end

	if itemString then
		args[i+1] = itemString
	end

	return args
end

function AlchemyList:Handle(argum, sender)
	if table.getn(self.db.account.Alchemy) == 0 then
		self:Respond(L["Database is currently empty."], sender)
		self:Print(sender..L[" wants to use AlchemyList but the Alchemy panel has not yet been opened."])
		return
	end

	if LOCKED then
		self:Respond(L["Wait ... ("]..self.db.account.TIMELOCK..L[" seconds lock (antispam))"], sender)
		return
	end

	local searchresult = {}

	for i in pairs(self.db.account.Alchemy) do
		local paf = 0
		local AlchemyName = string.lower(self.db.account.AlchemyName[i])
		local AlchemyDescription = string.lower(self.db.account.AlchemyDescription[i])
		local AlchemyReagent = string.lower(self.db.account.AlchemyReagent[i])
		local AlchemyMinLevel = self.db.account.MinLevel[i]
		local minLevel = 0
		if self.db.account.level then
			minLevel = self.db.account.level
		end

		for j in pairs(argum) do
			local arg = string.lower(argum[j])
			if strfind(AlchemyName, arg) and (AlchemyMinLevel >= minLevel or AlchemyMinLevel == 0) then
				paf = paf + 1
			elseif self.db.account.AlchemyRecipe and strfind(string.lower(self.db.account.AlchemyRecipe[i]), arg) and (AlchemyMinLevel >= minLevel or AlchemyMinLevel == 0) then
				paf = paf + 1
			elseif self.db.account.searchdesc and strfind(AlchemyDescription, arg) and (AlchemyMinLevel >= minLevel or AlchemyMinLevel == 0) then
				paf = paf + 1
			elseif strfind(AlchemyReagent, arg) and (AlchemyMinLevel >= minLevel or AlchemyMinLevel == 0) then
				paf = paf + 1
			end
		end

		if paf == table.getn(argum) then
			if self.db.account.AlchemyRecipe then
				table.insert(searchresult, self.db.account.AlchemyRecipe[i])
			else
				table.insert(searchresult, self.db.account.Alchemy[i])
			end
		end
	end

	if table.getn(searchresult) == 0 then
		self:Respond(L["Nothing found."], sender)
	else
		for i in pairs(searchresult) do
			if i > 15 then
				self:Respond(L["Maximum results reached. Be more specific by using !alchy flask, elixir, mana, heal or something similar."], sender)
				do break end
			end
			self:Respond(searchresult[i], sender)
		end
		self:Lock()
	end
end

function AlchemyList:Lock()
	LOCKED = true
	self:ScheduleEvent(function() LOCKED = false return end, self.db.account.TIMELOCK)
end


function AlchemyList:Respond(message, sender)
	if sender == "AlchemyListChatOutput" then
		self:Print(message)
	else
		SendChatMessage(chatprefix2.." "..message, "WHISPER", nil, sender)
	end
end

function AlchemyList:TRADE_SKILL_SHOW()
	if self.db.account.autoscan and self:isAlchemyPanel() then
		self:Scan("Alchemy")
	end
end

function AlchemyList:isAlchemyPanel()
	if not TradeSkillFrameTitleText then
		return false
	end
	local frameTitle = TradeSkillFrameTitleText:GetText()
	if not strfind(frameTitle, L["Alchemy"]) then
		return false
	end
	return true
end

function AlchemyList:ToggleAutoScan()
	self.db.account.autoscan = not self.db.account.autoscan
	if self.db.account.autoscan then
		self:RegisterEvent("TRADE_SKILL_SHOW")
	else
		self:UnregisterEvent("TRADE_SKILL_SHOW")
	end
end


function AlchemyList:getChatPrefix()
	return string.lower(self.db.account.chatprefix)
end

function AlchemyList:getReagents(id)
	local numReagents = GetTradeSkillNumReagents(id)
	local reagents = ""

	for i = 1, numReagents do
		local itemLink = GetTradeSkillReagentItemLink(id, i)
		if itemLink then
			local found, _, itemString = string.find(itemLink, "\124%x+\124H(item:%d+):.+\124h%[.+%]")
			reagents = reagents .. GetTradeSkillReagentInfo(id, i) .. ", " .. itemString .. ", "
		end
	end

	return reagents
end

function AlchemyList:Scan()
	if not self:isAlchemyPanel() then
		self:Print(L["Open your Alchemy panel first."])
		return
	end

	self.db.account.Alchemy = {}
	self.db.account.AlchemyRecipe = {}
	self.db.account.AlchemyName = {}
	self.db.account.AlchemyDescription = {}
	self.db.account.AlchemyReagent = {}
	self.db.account.MinLevel = {}
	local i = 1
	local skillName, skillType, _, isExpanded = GetTradeSkillInfo(i)

	while skillName and skillType do
		if skillType ~= "header" and skillName then
			-- add Alchemy to the tables
			local skillLink = GetTradeSkillItemLink(i) or ""
			T:SetTradeSkillItem(i)

			local	skillName,
						skillLink,
						skillRarity,
						skillLevel,
						skillMinLevel,
						skillType,
						skillSubType,
						skillCount,
						skillEquipLoc,
						skillTexture = GetItemInfo(skillLink)

			local skillDescription = T:GetLine(T:NumLines()) or ""
			local skillRecipe = GetTradeSkillRecipeLink(i)

			table.insert(self.db.account.Alchemy, skillLink)
			table.insert(self.db.account.AlchemyRecipe, skillRecipe)
			table.insert(self.db.account.AlchemyName, skillName)
			table.insert(self.db.account.AlchemyDescription, skillDescription)
			table.insert(self.db.account.AlchemyReagent, self:getReagents(i))
			table.insert(self.db.account.MinLevel, skillMinLevel)
		end

		i = i + 1
		skillName, skillType, _, isExpanded = GetTradeSkillInfo(i)
	end
end